#include <sys/types.h>
#include <regex.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <fnmatch.h>

#define LINEBUF 1024

#define MAXMATCHES 4
#define COMMENT_REGEX "[[:space:]]*#.*"
#define CMD_REGEX "^[[:space:]]*([^[:space:]]+)[[:space:]]+([^[:space:]]+)[[:space:]]+([^[:space:]]+)[[:space:]]*$"
regex_t comment, cmd;
regmatch_t matches[MAXMATCHES];

#define S_ROOT_PERM "ROOT"
#define S_USER_PERM "USER"
#define S_NOPASS_PERM "NOPASS"

#define ROOT_PERM 1
#define USER_PERM 2
#define NOPASS_PERM 3


void init_regex () {
	regcomp (&comment, COMMENT_REGEX, REG_EXTENDED);
	regcomp (&cmd, CMD_REGEX, REG_EXTENDED);
}

void close_regex () {
	regfree (&comment);
	regfree (&cmd);
}

void drop_comment (char *line) {
	//printf("1 %s\n", line);
	if (regexec (&comment, line, MAXMATCHES, matches, 0) == 0) {
		line [matches[0].rm_so] = '\0'; //drop it
		//printf("2 %s %d\n", line, matches[0].rm_so);
	}
}

int process_cmd (char *line, char *user, char *path) {
	char *userpat, *pathpat, *permpat;
	if (regexec (&cmd, line, MAXMATCHES, matches, 0) == 0) {
		line [matches[1].rm_eo] = '\0'; userpat = line+matches[1].rm_so;
		line [matches[2].rm_eo] = '\0'; pathpat = line+matches[2].rm_so;
		line [matches[3].rm_eo] = '\0'; permpat = line+matches[3].rm_so;
		printf ("%s %s %s\n", userpat, pathpat, permpat);
		
		if (fnmatch (userpat, user, 0)==0 && fnmatch (pathpat, path, 0)==0) {
			if (strcasecmp(permpat, S_ROOT_PERM)==0) return ROOT_PERM; 
			if (strcasecmp(permpat, S_USER_PERM)==0) return USER_PERM;
			if (strcasecmp(permpat, S_NOPASS_PERM)==0) return NOPASS_PERM;
		}
	}
	return 0;
}

int main (int argc, char *argv[]) {	
	//int i;
	//for (i=argc; i>0; i--) {
	//	printf ("%d %d %s\n", i, argc-i, argv[argc-i]);
	//}


	FILE *f;
	char line[LINEBUF];
	int perm;
	char *user = "root";
	char *path = "/path/to/some/app";

	f = fopen ("test.conf", "r");
	if (f) {
		init_regex();			
		while (fgets (line, LINEBUF, f) ) {
			if (line [strlen(line)-1] == '\n') line[strlen(line)-1] = '\0';
			drop_comment (line);
			if (line[0] == '\0') continue;
			if (perm = process_cmd (line, user, path)) {
				printf ("perm %d\n", perm);
				break;
			}
		}
		close_regex();		
		fclose (f);
	}
	
	return 0;
}